# 🎥 How to record a video with JavaScript [demo video available]

[![Demo video: How to record a video with JavaScript](https://img.youtube.com/vi/GPINZB8ENUQ/0.jpg)](https://www.youtube.com/watch?v=GPINZB8ENUQ)



## About this article

This article describes how to shoot a video from JavaScript using the [MediaStream Recording API](https://developer.mozilla.org/docs/Web/API/MediaStream_Recording_API). The related resources are shown below.

- [Demo](https://gist.githack.com/tatsuyasusukida/b21c4c7d73c5e3d91ab97d8c040bc48e/raw/index.html)
- [Demo video](https://www.youtube.com/watch?v=GPINZB8ENUQ)
- [Source code](https://gist.github.com/tatsuyasusukida/b21c4c7d73c5e3d91ab97d8c040bc48e?file=index.html#file-index-html)
- [日本語版 / Japanese version](https://zenn.dev/tatsuyasusukida/articles/aa6e061d8b5273)



## Workflow

The workflow is shown below.

1. Coding preparation
2. Coding
3. Operation check



## Coding preparation

Run the following commands in your terminal to prepare for coding.

```shell
mkdir javascript-media-video
cd javascript-media-video
touch index.html main.js
```



## Coding

### index.html

Open index.html in your editor and enter the following content.

[Click to go to index.html](#file-index-html)

### main.js

Open main.js in your editor and enter the following content.

[Click to go to main.js](#file-main-js)

The points are shown below.

1. Get the media stream to capture the video.
2. Check if recording in video / webm format is possible.
3. Create a media recorder.
4. Call the mediaRecorder.start method to start recording.
5. Call the mediaRecorder.stop method to end recording.
6. Add the dataavailable event handler of mediaRecorder to get the video data.



## Operation check

Open index.html in your web browser. The following command are useful for macOS.

```shell
open index.html
```

You will be asked to allow access to the camera and microphone, so click the "Allow" button.

Click the "Allow" button and then the "Start" button to start recording.

Click the "Stop" button to stop recording.

Click the play button of the recorded video to play the video.



## Conclusion

The MediaStream Recording API can be used not only for recording video, but also for recording audio.
For information on how to record audio using MediaStream Recording, see [How to record audio using MediaStream Recording](https://zenn.dev/tatsuyasusukida/articles/e9fc25bcdbd370), so if you are interested, please have a look. Thank you for reading!


---

## Express HTML

```html
<!DOCTYPE html>
<html lang="en">
  <head>
    <meta charset="UTF-8" />
    <meta http-equiv="X-UA-Compatible" content="IE=edge" />
    <meta name="viewport" content="width=device-width, initial-scale=1.0" />
    <title>
      title
    </title>
  </head>
  <body>
    <h1>title</h1>
    <div>
      <button type="button" id="buttonStart">Start</button>
      <button type="button" id="buttonStop" disabled>Stop</button>
      <button type="button" id="buttonUploadBlob" disabled>
        Upload (Blob)
      </button>
      <button type="button" id="buttonUploadDataURL" disabled>
        Upload (DataURL)
      </button>
    </div>
    <div>
      <video autoplay muted playsinline id="videoLive"></video>
    </div>
    <div>
      <video controls playsinline id="videoRecorded"></video>
    </div>
    <script>
      async function main() {
        const buttonStart = document.querySelector("#buttonStart");
        const buttonStop = document.querySelector("#buttonStop");
        const buttonUploadBlob = document.querySelector("#buttonUploadBlob");
        const buttonUploadDataURL = document.querySelector(
          "#buttonUploadDataURL"
        );
        const videoLive = document.querySelector("#videoLive");
        const videoRecorded = document.querySelector("#videoRecorded");
        let recordedVideoBlob = null;

        const stream = await navigator.mediaDevices.getUserMedia({
          video: true,
          audio: true,
        });

        videoLive.srcObject = stream;

        const mediaRecorder = new MediaRecorder(stream, {
          mimeType: "video/webm",
        });

        buttonStart.addEventListener("click", () => {
          mediaRecorder.start();
          buttonStart.setAttribute("disabled", "");
          buttonStop.removeAttribute("disabled");
        });

        buttonStop.addEventListener("click", () => {
          mediaRecorder.stop();
          buttonStart.removeAttribute("disabled");
          buttonStop.setAttribute("disabled", "");
          buttonUploadBlob.removeAttribute("disabled");
          buttonUploadDataURL.removeAttribute("disabled");
        });

        mediaRecorder.addEventListener("dataavailable", (event) => {
          videoRecorded.src = URL.createObjectURL(event.data);
          recordedVideoBlob = event.data;
        });

        buttonUploadBlob.addEventListener("click", async () => {
          try {
            const search = new URLSearchParams({
              extname: ".webm",
            }).toString();

            const url = "/api/upload/blob?" + search;

            console.log(recordedVideoBlob);

            const response = await fetch(url, {
              method: "POST",
              headers: {
                "Content-Type": "application/octet-stream",
              },
              body: recordedVideoBlob,
            });

            if (response.status !== 201) {
              console.warn(response.status);
              console.warn(await response.text());
            }
          } catch (err) {
            console.error(err);
          }
        });

        buttonUploadDataURL.addEventListener("click", async () => {
          try {
            const url = "/api/upload/dataurl";
            const response = await fetch(url, {
              method: "POST",
              headers: {
                "Content-Type": "application/json; charset=UTF-8",
              },
              body: JSON.stringify({
                extname: ".webm",
                dataUrl: await convertBlob(recordedVideoBlob),
              }),
            });

            if (response.status !== 201) {
              console.warn(response.status);
              console.warn(await response.text());
            }
          } catch (err) {
            console.error(err);
          }
        });
      }

      async function convertBlob(blob) {
        return await new Promise((resolve, reject) => {
          const fileReader = new FileReader();

          const subscribe = () => {
            fileReader.addEventListener("abort", onAbort);
            fileReader.addEventListener("error", onError);
            fileReader.addEventListener("load", onLoad);
          };

          const unsubscribe = () => {
            fileReader.removeEventListener("abort", onAbort);
            fileReader.removeEventListener("error", onError);
            fileReader.removeEventListener("load", onLoad);
          };

          const onAbort = () => {
            unsubscribe();
            reject(new Error("abort"));
          };

          const onError = (event) => {
            unsubscribe();
            reject(event.target.error);
          };

          const onLoad = (event) => {
            unsubscribe();
            resolve(event.target.result);
          };

          subscribe();
          fileReader.readAsDataURL(blob);
        });
      }

      main();
    </script>
  </body>
</html>
```


## Express Server

```javascript
const express = require("express");
const path = require("path");
const fsPromises = require("fs/promises");

const app = express();

app.get("/", (req, res) => {
  res.sendFile(path.join(__dirname, "index.html"));
});

app.post("/api/upload/blob", async (req, res, next) => {
  try {
    const rawBody = await new Promise((resolve, reject) => {
      const chunks = [];

      req.on("data", (chunk) => chunks.push(chunk));
      req.on("end", () => resolve(Buffer.concat(chunks)));
      req.on("error", (err) => reject(err));
    });

    const outputDirectory = path.join(__dirname, "tmp");
    const outputFilename = Date.now() + req.query.extname;
    const outputPath = path.join(outputDirectory, outputFilename);

    await fsPromises.mkdir(outputDirectory, { recursive: true });
    await fsPromises.writeFile(outputPath, rawBody);

    res.status(201).end();
  } catch (err) {
    next(err);
  }
});

app.post("/api/upload/dataurl", express.json(), async (req, res, next) => {
  try {
    const recordedVideoBuffer = Buffer.from(
      req.body.dataUrl.split("base64,")[1],
      "base64"
    );

    const outputDirectory = path.join(__dirname, "tmp");
    const outputFilename = Date.now() + req.body.extname;
    const outputPath = path.join(outputDirectory, outputFilename);

    await fsPromises.mkdir(outputDirectory, { recursive: true });
    await fsPromises.writeFile(outputPath, recordedVideoBuffer);

    res.status(201).end();
  } catch (err) {
    next(err);
  }
});

app.listen(3000);

```


## Another idea

```javascript
async function main () {
  const buttonStart = document.querySelector('#buttonStart')
  const buttonStop = document.querySelector('#buttonStop')
  const videoLive = document.querySelector('#videoLive')
  const videoRecorded = document.querySelector('#videoRecorded')

  const stream = await navigator.mediaDevices.getUserMedia({ // <1>
    video: true,
    audio: true,
  })

  videoLive.srcObject = stream

  if (!MediaRecorder.isTypeSupported('video/webm')) { // <2>
    console.warn('video/webm is not supported')
  }

  const mediaRecorder = new MediaRecorder(stream, { // <3>
    mimeType: 'video/webm',
  })

  buttonStart.addEventListener('click', () => {
    mediaRecorder.start() // <4>
    buttonStart.setAttribute('disabled', '')
    buttonStop.removeAttribute('disabled')
  })

  buttonStop.addEventListener('click', () => {
    mediaRecorder.stop() // <5>
    buttonStart.removeAttribute('disabled')
    buttonStop.setAttribute('disabled', '')
  })

  mediaRecorder.addEventListener('dataavailable', event => {
    videoRecorded.src = URL.createObjectURL(event.data) // <6>
  })
}

main()
```